@@ -1,4 +1,8 @@
-dataf.geneexp <- function() return(structure(list(dataf = list(structure(list(args = c(1, 2, 3, 
+dataf.geneexp <- function() return(structure(list(
+  name = "Gene expression profile",
+  args = "Time",
+  vals = "Gene Expression Level",
+  dataf = list(structure(list(args = c(1, 2, 3, 
 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 
 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 
 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 
@@ -1017,6 +1021,5 @@ dataf.geneexp <- function() return(structure(list(dataf = list(structure(list(ar
     3, 3, 2, 3, 3, 3, 3, 3, 1, 3, 1, 1, 3, 1, 3, 1, 3, 1, 3, 
     3, 2, 1, 3, 3, 3, 3, 1, 2, 2, 2, 2, 2, 1, 2, 2, 3, 3, 2, 
     1, 3, 1, 2, 2, 2, 3, 3, 1, 2, 1, 1, 1, 2, 2, 3, 2, 2, 2, 
-    2, 1, 2, 2, 3, 3, 3, 3, 2)), .Names = c("dataf", "labels"
-))
+    2, 1, 2, 2, 3, 3, 3, 3, 2)))
 )
\ No newline at end of file
